[![License](https://img.shields.io/badge/license-MIT-blue.svg)](LICENSE)
[![Nim](https://img.shields.io/badge/Nim-gold.svg?logo=nim&logoColor=black)](https://nim-lang.org)

# GrabNim

GrabNim is a simple tool to install and switch between different versions of the Nim compiler.

## Features

- Install pre-built Nim releases (stable builds from nim-lang.org and nightly builds from github)
- Install Nim from source (supports tagged versions and `devel` for master branch)
- Switch between installed versions
- Uses symlinks on Unix-like OSes and junctions on Windows
- Doesn't litter in Home directory, respects [XDG spec](https://specifications.freedesktop.org/basedir-spec/latest/)

## Usage

```
Usage:
  grabnim                     Install and switch to latest stable version of compiler
  grabnim <version>           Install and switch to specific version of compiler
  grabnim compile <version>   Clone Nim repo, compile, install and switch to this version

  grabnim update  <version>   Force update installed compiler (only 'devel' or '2.2.x')
  grabnim upd     <version>   Short version of 'update'

  grabnim delete  <version>   Delete local version of compiler
  grabnim del     <version>   Short version of 'delete'

  grabnim fetch               List remote available versions
  grabnim list                List local installed versions
  grabnim help                Display this help message

Examples:
  grabnim 2.2.4             Install Nim v2.2.4 from official website
  grabnim devel             Install Nightly Nim devel from github
  grabnim 2.2.x             Install Nightly Nim latest-2-2 from github
  grabnim compile devel     Clone Nim repo and build master branch of the Nim compiler
  grabnim compile 2.0.8     Clone Nim repo, switch to 'v2.0.8' tag and build Nim compiler
  grabnim update devel      Reset installed Nim repo, pull new commits and build Nim compiler
  grabnim update 2.2.x      Delete and Reinstall Nightly Nim latest-2-2 from github
```

## Paths Used by GrabNim

**Cache Directory** (cached nim-repo and csources):
- **Windows:** System `%TEMP%` (e.g. `C:\Users\YourUser\AppData\Local\Temp`)
- **Linux/macOS:** `$XDG_CACHE_HOME/grabnim-cache` or `~/.cache/grabnim-cache` if `XDG_CACHE_HOME` is not set

**Grabnim Directory** (installed Nim versions):
- **Windows:** `%LOCALAPPDATA%\grabnim` (e.g., `C:\Users\YourUser\AppData\Local\grabnim`)
- **Linux/macOS:** `$XDG_DATA_HOME/grabnim` or `~/.local/share/grabnim` if `XDG_DATA_HOME` is not set

*Each version lives in its own subfolder (e.g., `nim-2.0.0`, `nim-devel`)*

## Version Switching:

- Grabnim creates a `current` symlink (junction on Windows) in the Grabnim Directory
- Symlink points to the selected version folder (e.g., `grabnim/current` → `grabnim/nim-2.0.0`)
- When you add `.../grabnim/current/bin` to your PATH, it'll always point to currently active Nim version binaries (compiler and tools).

## Installation

1. Grab a binary from [releases](https://codeberg.org/janakali/grabnim/releases/latest)
2. Add GrabNim directory to 'PATH' env ([see this gist for instructions](https://gist.github.com/janAkali/338e6bf143fcd0161a2f0d768b938e4b)):
  > Place the GrabNim path *after* your Nimble path in your `PATH` environment variable.
  > This ensures that Nimble-installed packages take precedence over the versions bundled with the Nim compiler.

  ##### Path Examples:
  - Linux:
      ```bash
      export PATH="$HOME/.local/share/grabnim/current/bin:$PATH"
      export PATH="$HOME/.nimble/bin:$PATH"
      # order might look wrong, but it is correct: "<nimble>:<grabnim>:$PATH"
      ```
  - Windows:
      ```bat
      %USERPROFILE%\.nimble\bin
      %LOCALAPPDATA%\grabnim\current\bin
      ```
  - MacOS:
      ```bash
      $HOME/.nimble/bin
      $HOME/.local/share/grabnim/current/bin
      ```

3. (Optional) Move binary or create symlink to directory that's in PATH (e.g. `/usr/local/bin` or `$HOME/.nimble/bin`).

## Building GrabNim

### Prerequisites

- Nim compiler v2.0.0 or later
- Basic build tools (make, gcc, etc.)
- Git

1. Clone the repository:
   ```bash
   git clone https://codeberg.org/janakali/grabnim
   cd grabnim
   ```

2. Build with Nim:
   ```bash
   nim c -d:release grabnim
   ```

## License

This project is licensed under the MIT License. See the [LICENSE](LICENSE) file for details.
